# -*- coding: utf-8 -*-

import os

test_runner = None
old_config = None

os.environ['DJANGO_SETTINGS_MODULE'] = 'tests.settings'

import django
if hasattr(django, 'setup'):
    django.setup()


def setup():
    from django.test.runner import DiscoverRunner as TestSuiteRunner
    global test_runner
    global old_config

    test_runner = TestSuiteRunner()
    test_runner.setup_test_environment()
    old_config = test_runner.setup_databases()


def teardown():
    test_runner.teardown_databases(old_config)
    test_runner.teardown_test_environment()



# -*- coding: utf-8 -*-

from __future__ import absolute_import, unicode_literals

from django.core.exceptions import ImproperlyConfigured
from django.test import TestCase, override_settings

from mongoengine.connection import _connections, get_connection

from mongo_connection.loading import ConnectionHandler, ConnectionWrapper


class ConnectionHandlerTestCase(TestCase):

    def test_class_init(self):
        handler = ConnectionHandler({})
        self.assertEqual(handler.connections_info, {})

        handler = ConnectionHandler({
            'default': {
                'NAME': 'default',
                'HOST': 'localhost'
            }
        })
        self.assertEqual(handler.connections_info, {
            'default': {
                'NAME': 'default',
                'HOST': 'localhost'
            }
        })

    def test_get_all_connections(self):
        handler = ConnectionHandler({
            'default': {
                'NAME': 'default',
                'HOST': 'localhost'
            },
            'other': {
                'NAME': 'other',
                'HOST': 'localhost'
            }
        })

        for wrapper in map(lambda e: repr(e).strip('<>').split(' object at ')[0], handler.all()):
            self.assertEqual(wrapper, 'mongo_connection.loading.ConnectionWrapper')

    def test_get_item(self):
        handler = ConnectionHandler({
            'default': {
                'NAME': 'default',
                'HOST': 'localhost'
            }
        })
        wrapper = handler['default']
        self.assertIsInstance(wrapper, ConnectionWrapper)

    def test_get_item_same_instance(self):
        handler = ConnectionHandler({
            'default': {
                'NAME': 'default',
                'HOST': 'localhost'
            }
        })

        # Make sure we get the same item when getting by same alias
        path1, address1 = repr(handler['default']).strip('<>').split(' object at ')
        path2, address2 = repr(handler['default']).strip('<>').split(' object at ')
        self.assertEqual(path1, path2)
        self.assertEqual(address1, address2)

    def test_get_item_invalid_alias(self):
        handler = ConnectionHandler({})
        try:
            handler['default']
            self.fail('Did not fail when trying to get a non-existing connection alias.')
        except ImproperlyConfigured as e:
            self.assertEqual(str(e), 'The key "default" isn\'t an available connection.')

    def test_reload(self):
        handler = ConnectionHandler({
            'default': {
                'NAME': 'default',
                'HOST': 'localhost'
            }
        })

        # Make sure we get a new object when reloaded
        self.assertNotEqual(handler['default'], handler.reload('default'))

    def test_reload_invalid_alias(self):
        handler = ConnectionHandler({
            'default': {
                'NAME': 'default',
                'HOST': 'localhost'
            }
        })

        try:
            handler.reload('slave')
        except ImproperlyConfigured as e:
            self.assertEqual(str(e), 'The key "slave" isn\'t an available connection.')
        else:
            self.fail('Should fail with ImproperlyConfigured when reloading an invalid alias.')


@override_settings(MONGO_CONNECTIONS={
    'default': {
        'NAME': 'default',
        'HOST': 'localhost',
    }
})
class ConnectionWrapperTestCase(TestCase):

    def test_is_connected_by_default(self):
        wrapper = ConnectionWrapper(using='default')
        self.assertTrue(wrapper.is_connected)

    def test_reset_connection(self):
        wrapper = ConnectionWrapper(using='default')
        wrapper.reset_connection()
        self.assertFalse(wrapper.is_connected)

    def test_connect(self):
        wrapper = ConnectionWrapper(using='default')
        wrapper.reset_connection()
        self.assertFalse(wrapper.is_connected)

        wrapper.connect()
        self.assertTrue(wrapper.is_connected)

    def test_automatic_reconnect_when_reading_property(self):
        wrapper = ConnectionWrapper(using='default')

        wrapper.reset_connection()
        self.assertFalse(wrapper.is_connected)

        wrapper.connection
        self.assertTrue(wrapper.is_connected)

    def test_wrapper_register_connection(self):
        wrapper = ConnectionWrapper(using='default')
        self.assertTrue(wrapper.using in _connections)
        self.assertEqual(get_connection(wrapper.using), wrapper.connection)



# -*- coding: utf-8 -*-

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from .constants import DEFAULT_ALIAS
from .loading import ConnectionHandler

__title__ = 'django-mongo-connection'
__version__ = '0.0.1'
__author__ = 'Rolf Haavard Blindheim'
__license__ = 'MIT License'

VERSION = __version__


default_app_config = 'mongo_connection.apps.MongoConnectionConfig'

if not hasattr(settings, 'MONGO_CONNECTIONS'):
    raise ImproperlyConfigured('The MONGO_CONNECTIONS setting is required.')

if DEFAULT_ALIAS not in settings.MONGO_CONNECTIONS:
    raise ImproperlyConfigured('The default alias "%s" must be included '
                               'in the MONGO_CONNECTIONS setting.' % DEFAULT_ALIAS)

# Load connections
connections = ConnectionHandler(settings.MONGO_CONNECTIONS)



# -*- coding: utf-8 -*-

from django.apps import AppConfig

from . import connections


class MongoConnectionConfig(AppConfig):
    name = 'mongo_connection'
    label = 'mongo_connection'

    def ready(self):
        """
        Reload all mongo connections.
        """
        connections.reload_all()



# -*- coding: utf-8 -*-

from django.test import TestCase

from mongoengine.connection import DEFAULT_CONNECTION_NAME
from mongo_connection.constants import DEFAULT_ALIAS


class ConstantsTestCase(TestCase):

    def test_default_alias(self):
        self.assertEqual(DEFAULT_ALIAS, DEFAULT_CONNECTION_NAME)



# -*- coding: utf-8 -*-

import re
import os

try:
    from setuptools import setup
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)

setup(
    name='django-mongo-connection',
    version=get_version('mongo_connection'),
    description='Simple connection wrapper for Mongoengine',
    long_description='Define Mongoengine connections in settings.py and make sure they\'re available when needed.',
    author='Rolf Håvard Blindheim',
    author_email='rhblind@gmail.com',
    url='https://github.com/inonit/django-mongo-connection',
    download_url='https://github.com/inonit/django-mongo-connection.git',
    license='MIT License',
    packages=[
        'mongo_connection',
    ],
    include_package_data=True,
    install_requires=[
        'Django>=1.8.0',
        'mongoengine>=0.10.6'
    ],
    tests_require=[
        'nose',
        'coverage'
    ],
    zip_safe=False,
    test_suite='tests.run_tests.start',
    classifiers=[
        'Operating System :: OS Independent',
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)



# -*- coding: utf-8 -*-

import threading

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from mongoengine import connect
from mongoengine.connection import _connections, disconnect

from .constants import DEFAULT_ALIAS


class ConnectionHandler(object):
    """
    Manage connections for the local thread.
    """

    def __init__(self, connections_info):
        self.connections_info = connections_info
        self.thread_local = threading.local()

    def __getitem__(self, item):
        if not hasattr(self.thread_local, 'connections'):
            self.thread_local.connections = {}
        elif item in self.thread_local.connections:
            return self.thread_local.connections[item]

        self.ensure_defaults(item)
        self.thread_local.connections[item] = ConnectionWrapper(using=item)
        return self.thread_local.connections[item]

    def all(self):
        return [self[alias] for alias in self.connections_info]

    def ensure_defaults(self, alias):
        try:
            self.connections_info[alias]
        except KeyError:
            raise ImproperlyConfigured('The key "%s" isn\'t an available connection.' % alias)

    def reload(self, alias):
        if not hasattr(self.thread_local, 'connections'):
            self.thread_local.connections = {}
        try:
            del self.thread_local.connections[alias]
        except KeyError:
            pass

        return self.__getitem__(alias)

    def reload_all(self):
        for alias in settings.MONGO_CONNECTIONS.keys():
            self.reload(alias)


class ConnectionWrapper(object):

    def __init__(self, using=None):
        self.using = using or DEFAULT_ALIAS
        self.options = settings.MONGO_CONNECTIONS.get(self.using, {})
        self._connection = None

        # Make an initial connection
        self.connect()

    @property
    def is_connected(self):
        return self._connection is not None and self.using in _connections

    @property
    def connection(self):
        if self._connection is None:
            self._connection = self.connect()
        return self._connection

    def connect(self):
        self._connection = connect(alias=self.using,
                                   tz_aware=settings.USE_TZ,
                                   db=self.options.get('NAME', None),
                                   host=self.options.get('HOST', None),
                                   port=self.options.get('PORT', None),
                                   username=self.options.get('USERNAME', None),
                                   password=self.options.get('PASSWORD', None))
        return self.connection

    def reset_connection(self):
        disconnect(self.using)
        self._connection = None
